/**
 * 
 */
package edu.wisc.uwss;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.util.DigestUtils;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Implementation of {@link UWUserDetails} returned by the demo services in this package.
 * 
 * @author Nicholas Blair
 */
public class UWUserDetailsImpl extends User implements UWUserDetails, HasModifiableSource {

  /**
   * 
   */
  private static final long serialVersionUID = -8120331081953321708L;

  private String eppn;
  private final String pvi;
  private final String fullName;
  private final Collection<String> uddsMembership;
  private final String emailAddress;
  private final String emailAddressHash;
  private String source;
  private String customLogoutUrl;
  private String isisEmplid;
  private String firstName;
  private String lastName;
  /**
   * 
   * @param pvi
   * @param username
   * @param password
   * @param fullName
   * @param emailAddress
   */
  public UWUserDetailsImpl(String pvi, String username, String password,
      String fullName, String emailAddress) {
    this(pvi, username, password, fullName, emailAddress, Collections
        .<String> emptyList());
  }
  
  /**
   * The primary constructor for {@link UWUserDetails}.
   * @param pvi
   * @param username
   * @param password
   * @param fullName
   * @param emailAddress
   * @param uddsMembership
   * @param grantedAuthorities
   * 
   * No other constructors should be annotated with JsonCreator.  
   * @see http://stackoverflow.com/questions/15931082/how-to-deserialize-a-class-with-overloaded-constructors-using-jsoncreator
   */
  @JsonCreator
  public UWUserDetailsImpl(@JsonProperty("pvi") String pvi, @JsonProperty("username") String username, @JsonProperty("password") String password,
      @JsonProperty("fullName") String fullName, @JsonProperty("emailAddress") String emailAddress,
      @JsonProperty("uddsMembership") Collection<String> uddsMembership, @JsonProperty("authorities") Collection<? extends GrantedAuthority> authorities) {
    super(username, password, authorities);
    this.pvi = pvi;
    this.fullName = fullName;
    this.uddsMembership = uddsMembership;
    this.emailAddress = StringUtils.isNotBlank(emailAddress) ?
        emailAddress.trim().toLowerCase() : emailAddress;
    this.emailAddressHash = StringUtils.isNotBlank(emailAddress) ? DigestUtils
        .md5DigestAsHex(this.emailAddress.getBytes()) : null;
  }

  /**
   * @param pvi
   * @param username
   * @param password
   * @param fullName
   * @param uddsMembership
   */
  public UWUserDetailsImpl(String pvi, String username, String password,
       String fullName,  String emailAddress, Collection<String> uddsMembership) {
    this(pvi, username, password, fullName, emailAddress, uddsMembership, Collections.<GrantedAuthority>emptyList());
  }

  /**
   * Clone style constructor.
   * 
   * @param original
   */
  public UWUserDetailsImpl(UWUserDetails original) {
    this(original.getPvi(), original.getUsername(), original.getPassword(), original
        .getFullName(), original.getEmailAddress(), original
        .getUddsMembership(), original.getAuthorities());
    setCustomLogoutUrl(original.getCustomLogoutUrl());
    setEppn(original.getEppn());
    setIsisEmplid(original.getIsisEmplid());
    setSource(original.getSource());
  }

  /* (non-Javadoc)
   * @see edu.wisc.uwss.UWUserDetails#getPvi()
   */
  @Override
  public String getPvi() {
    return this.pvi;
  }

  public String getFullName() {
    return this.fullName;
  }

  public Collection<String> getUddsMembership() {
    return uddsMembership;
  }

  public String getSource() {
    return source;
  }
  /**
   * @param source the source to set
   */
  @Override
  public void setSource(String source) {
    this.source = source;
  }
  
  public String getCustomLogoutUrl() {
    return customLogoutUrl;
  }
  /**
   * @param customLogoutUrl the customLogoutUrl to set
   */
  public void setCustomLogoutUrl(String customLogoutUrl) {
    this.customLogoutUrl = customLogoutUrl;
  }

  public String getEmailAddress() {
    return this.emailAddress;
  }

  /**
   * Returns MD5 hash of {@link #getEmailAddress()} in Hex.
   * 
   * @return the MD5 hash of {@link #getEmailAddress()} if not blank,
   *         otherwise null
   */
  public final String getEmailAddressHash() {
    return this.emailAddressHash;
  }
  /**
   * @return the emplid
   */
  public String getIsisEmplid() {
    return isisEmplid;
  }
  /**
   * @param isisEmplid the ISIS emplid to set
   */
  public void setIsisEmplid(String isisEmplid) {
    this.isisEmplid = isisEmplid;
  }
  /**
   * {@inheritDoc}
   * 
   * Copied super's toString, but modified how {@link #getAuthorities()} is
   * represented.
   */
  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("PVI: ").append(getPvi()).append("; ");
    sb.append("Username: ").append(getUsername()).append("; ");
    sb.append("Password: [PROTECTED]; ");
    sb.append("Enabled: ").append(isEnabled()).append("; ");
    sb.append("AccountNonExpired: ").append(isAccountNonExpired())
    .append("; ");
    sb.append("credentialsNonExpired: ").append(isCredentialsNonExpired())
    .append("; ");
    sb.append("AccountNonLocked: ").append(isAccountNonLocked())
    .append("; ");
    sb.append("Authorities: ")
    .append(Arrays.toString(getAuthorities().toArray()))
    .append("; ");
    sb.append("Source: ").append(getSource()).append("; ");
    sb.append("Emplid: ").append(getIsisEmplid()).append("; ");
    sb.append("Custom Logout Url: ").append(getCustomLogoutUrl());
    
    return sb.toString();
  }

  /**
   * @return the eppn
   */
  @Override
  public String getEppn() {
    return eppn;
  }

  /**
   * @param eppn the eppn to set
   */
  public void setEppn(String eppn) {
    this.eppn = eppn;
  }
  /**
   * @return the firstName
   */
  public String getFirstName() {
    return firstName;
  }
  /**
   * @param firstName the firstName to set
   */
  public UWUserDetailsImpl setFirstName(String firstName) {
    this.firstName = firstName;
    return this;
  }
  /**
   * @return the lastName
   */
  public String getLastName() {
    return lastName;
  }
  /**
   * @param lastName the lastName to set
   */
  public UWUserDetailsImpl setLastName(String lastName) {
    this.lastName = lastName;
    return this;
  }

}
