/**
 * 
 */
package edu.wisc.uwss.preauth;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.net.URL;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.junit.Test;
import org.springframework.mock.web.MockHttpServletRequest;

import edu.wisc.uwss.UWUserDetails;
import edu.wisc.uwss.preauth.PreauthenticatedUserDetailsAttributeMapper.Default;

/**
 * Tests for {@link Default}.
 * 
 * @author Nicholas Blair
 */
public class PreauthenticatedUserDetailsAttributeMapperTest {

  private PreauthenticatedUserDetailsAttributeMapper.Default filter = new PreauthenticatedUserDetailsAttributeMapper.Default();
  
  /**
   * Verify expected behaviour for {@link PreauthenticatedUserDetailsAttributeMapper#mapUser(HttpServletRequest)}
   */
  @Test
  public void mapUser_success() {
    MockHttpServletRequest request = new MockHttpServletRequest();
    String uid = "somebody";
    String eppn = "somebody@wisc.edu";
    String pvi = "1234567";
    String name = "some body";
    String email = "some.body@wisc.edu";
    String emplid = "0000123456";
    List<String> uddsMembership = Collections.singletonList("udds1234");
    
    request.addHeader("eppn", eppn);
    request.addHeader("wiscedupvi", pvi);
    request.addHeader("uid", uid);
    request.addHeader("cn", name);
    request.addHeader("mail", email);
    request.addHeader("wisceduudds", uddsMembership);
    request.addHeader("wisceduisisemplid", emplid);
    request.addHeader("Shib-Identity-Provider", "https://logintest.wisc.edu/idp/shibboleth");
    
    UWUserDetails result = filter.mapUser(request);
    
    assertNotNull(result);
    assertEquals(uid, result.getUsername());
    assertEquals(eppn, result.getEppn());
    assertEquals(pvi, result.getPvi());
    assertEquals(name, result.getFullName());
    assertEquals(email, result.getEmailAddress());
    assertEquals(uddsMembership, result.getUddsMembership()); 
    assertEquals(emplid, result.getIsisEmplid());
    assertEquals("/Shibboleth.sso/Logout?return=https://logintest.wisc.edu/logout/", result.getCustomLogoutUrl());
  }
  
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * null input.
   */
  @Test
  public void toCustomLogoutUrl_null() {
    assertNull(filter.toCustomLogoutUrl(null));
  }
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * empty input.
   */
  @Test
  public void toCustomLogoutUrl_empty() {
    assertNull(filter.toCustomLogoutUrl(""));
  }
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * invalid input.
   */
  @Test
  public void toCustomLogoutUrl_invalid() {
    assertNull(filter.toCustomLogoutUrl("foo"));
  }
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * a valid input with no {@link URL#getPath()}.
   */
  @Test
  public void toCustomLogoutUrl_no_path() {
    assertEquals("/Shibboleth.sso/Logout?return=https://somewhere.wisc.edu/logout/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu"));
  }
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * expected valid input.
   */
  @Test
  public void toCustomLogoutUrl_success() {
    assertEquals("/Shibboleth.sso/Logout?return=https://somewhere.wisc.edu/logout/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu/idp/shibboleth"));
  }
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * expected valid input and a different customLogoutPrefix.
   */
  @Test
  public void toCustomLogoutUrl_alternate_prefix() {
    filter.setCustomLogoutPrefix("/Chibbolath.sso/Logout?return=");
    assertEquals("/Chibbolath.sso/Logout?return=https://somewhere.wisc.edu/logout/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu/idp/shibboleth"));
  }
  /**
   * Verify behavior of {@link Default#toCustomLogoutUrl(String)} for
   * expected valid input and a different customLogoutSuffix.
   */
  @Test
  public void toCustomLogoutUrl_alternate_suffix() {
    filter.setCustomLogoutSuffix("/luguot/");
    assertEquals("/Shibboleth.sso/Logout?return=https://somewhere.wisc.edu/luguot/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu/idp/shibboleth"));
  }
}
